package de.hdm_stuttgart.mi.pangram;

/**
 * <p>Checking whether a given ASCII String is a pangram.</p>
 */
public class PangramChecker {

    static private final byte alphabetCharCount = 'z' - 'a' + 1; // 26 characters {'a', 'b', ..., 'z'}
    static private final char alphabetStartIndex = 'a';

    /**
     * <p>Is a given string a pangram with respect to the set of characters {'a', 'b', ..., 'z'} or not?</p>
     *
     * @param s String to be evaluated.
     * @return true if string is a pangram, false otherwise.
     */
    static public boolean isPangram(final String s) {
        final boolean[] charactersFound =       // Reserve one boolean per alphabet character, initialized to false.
                new boolean[alphabetCharCount];

        int distinctCharacterCount = 0;         // Number of different characters found so far.

        for (final char c : s.toLowerCase().toCharArray()) {
            if ('a' <= c && c <= 'z'                // Character belongs to alphabet?
                    && !charactersFound[c - 'a']) { // Character already discovered during call?
                distinctCharacterCount++;           // Yet another distinct one.
                charactersFound[c - 'a'] = true;    // Mark alphabet character as being found.
            }
        }
        return distinctCharacterCount == alphabetCharCount;  // Did we find all 26?
    }

    /**
     * Same as {@link #isPangram(String)} using a stream based implementation
     * @param s See {@link #isPangram(String)}
     * @return See {@link #isPangram(String)}
     */
    static public boolean isPangram2(final String s) {
        return alphabetCharCount == s.toLowerCase().chars()
                .filter(i -> i >= 'a' && i <= 'z')
                .distinct()
                .count();
    }

    /**
     * Main entry point
     * @param args Unused.
     */
    public static void main(String[] args) {
        System.out.println(isPangram("The quick brown fox jumps over the lazy dog"));
        System.out.println(isPangram("The quick brown fox jumps over the lazy"));
    }
}
